==> workos-rust-0.1.0.crate: workos-rust-0.1.0/src/client.rs <==
use crate::error::WorkOSError;
use crate::models::{Group, Organization, User};
use futures::stream::{self};
use futures::{StreamExt, TryStreamExt};
use std::collections::HashMap;
use surf::StatusCode;

const DEFAULT_BASE_URL: &str = "https://api.workos.com";

#[derive(Clone)]
pub struct Client {
    base_url: String,
    api_key: String,
}

impl Client {
    pub fn new(custom_base_url: Option<String>, api_key: String) -> Self {
        Client {
            base_url: custom_base_url.unwrap_or_else(|| DEFAULT_BASE_URL.to_string()),
            api_key,
        }
    }
}

impl Client {
    pub async fn get_user(&self, user_id: &str) -> Result<User, WorkOSError> {
        let mut res = surf::get(format!("{}/directory_users/{}", &self.base_url, user_id))
            .header("Authorization", format!("Bearer {}", &self.api_key))
            .await
            .map_err(|_| WorkOSError::Unknown)?;

        match res.status() {
            StatusCode::Ok => Ok(()),
            StatusCode::NotFound => Err(WorkOSError::NotFound(user_id.to_string())),
            _ => Err(WorkOSError::Unknown),
        }?;

        res.body_json().await.map_err(|_| WorkOSError::Unknown)
    }

    pub async fn get_users(
        &self,
        user_ids: Vec<String>,
    ) -> Result<HashMap<String, User>, WorkOSError> {
        stream::iter(user_ids)
            .then(|id| async move {
                let user = self.get_user(&id).await?;
                Ok::<User, WorkOSError>(user)
            })
            .try_fold(HashMap::new(), |mut users, user| async move {
                users.insert(user.id.clone(), user);
                Ok(users)
            })
            .await
    }

    pub async fn get_group(&self, group_id: &str) -> Result<Group, WorkOSError> {
        let mut res = surf::get(format!("{}/directory_groups/{}", &self.base_url, group_id))
            .header("Authorization", format!("Bearer {}", &self.api_key))
            .await
            .map_err(|_| WorkOSError::Unknown)?;

        match res.status() {
            StatusCode::Ok => Ok(()),
            StatusCode::NotFound => Err(WorkOSError::NotFound(group_id.to_string())),
            _ => Err(WorkOSError::Unknown),
        }?;

        res.body_json().await.map_err(|_| WorkOSError::Unknown)
    }

    pub async fn get_groups(
        &self,
        group_ids: Vec<String>,
    ) -> Result<HashMap<String, Group>, WorkOSError> {
        stream::iter(group_ids)
            .then(|id| async move {
                let group = self.get_group(&id).await?;
                Ok::<Group, WorkOSError>(group)
            })
            .try_fold(HashMap::new(), |mut groups, group| async move {
                groups.insert(group.id.clone(), group);
                Ok(groups)
            })
            .await
    }

    pub async fn get_organization(
        &self,
        organization_id: String,
    ) -> Result<Organization, WorkOSError> {
        let mut res = surf::get(format!(
            "{}/organizations/{}",
            &self.base_url, organization_id
        ))
        .header("Authorization", format!("Bearer {}", &self.api_key))
        .await
        .map_err(|_| WorkOSError::Unknown)?;

        match res.status() {
            StatusCode::Ok => Ok(()),
            StatusCode::NotFound => Err(WorkOSError::NotFound(organization_id.to_string())),
            _ => Err(WorkOSError::Unknown),
        }?;

        res.body_json().await.map_err(|_| WorkOSError::Unknown)
    }
}

==> workos-rust-0.2.0.crate: workos-rust-0.2.0/src/client.rs <==
use crate::directory_sync::{Group, User};
use crate::error::WorkOSError;
use crate::organizations::Organization;
use anyhow::{anyhow, Context, Result};
use futures::stream::{self};
use futures::{StreamExt, TryStreamExt};
use std::collections::HashMap;
use surf::StatusCode;

const DEFAULT_BASE_URL: &str = "https://api.workos.com";

#[derive(Clone)]
pub struct Client {
    base_url: String,
    api_key: String,
}

impl Client {
    pub fn new(custom_base_url: Option<String>, api_key: String) -> Self {
        Client {
            base_url: custom_base_url.unwrap_or_else(|| DEFAULT_BASE_URL.to_string()),
            api_key,
        }
    }
}

impl Client {
    pub async fn get_user(&self, user_id: &str) -> Result<User, WorkOSError> {
        let mut res = surf::get(format!("{}/directory_users/{}", &self.base_url, user_id))
            .header("Authorization", format!("Bearer {}", &self.api_key))
            .await
            .map_err(|e| anyhow!(e))
            .context("unable to perform request")
            .map_err(WorkOSError::Unknown)?;

        match res.status() {
            StatusCode::Ok => Ok(()),
            StatusCode::NotFound => Err(WorkOSError::NotFound(user_id.to_string())),
            _ => Err(WorkOSError::Unknown(anyhow!(format!(
                "received unexpected status '{}'",
                res.status()
            )))),
        }?;

        res.body_json()
            .await
            .map_err(|e| anyhow!(e))
            .context("unable to parse response")
            .map_err(WorkOSError::Unknown)
    }

    pub async fn get_users(
        &self,
        user_ids: Vec<String>,
    ) -> Result<HashMap<String, User>, WorkOSError> {
        stream::iter(user_ids)
            .then(|id| async move {
                let user = self.get_user(&id).await.context("unable to get user")?;
                Ok::<User, WorkOSError>(user)
            })
            .try_fold(HashMap::new(), |mut users, user| async move {
                users.insert(user.id.clone(), user);
                Ok(users)
            })
            .await
    }

    pub async fn get_group(&self, group_id: &str) -> Result<Group, WorkOSError> {
        let mut res = surf::get(format!("{}/directory_groups/{}", &self.base_url, group_id))
            .header("Authorization", format!("Bearer {}", &self.api_key))
            .await
            .map_err(|e| anyhow!(e))
            .context("unable to perform request")
            .map_err(WorkOSError::Unknown)?;

        match res.status() {
            StatusCode::Ok => Ok(()),
            StatusCode::NotFound => Err(WorkOSError::NotFound(group_id.to_string())),
            _ => Err(WorkOSError::Unknown(anyhow!(format!(
                "received unexpected status '{}'",
                res.status()
            )))),
        }?;

        res.body_json()
            .await
            .map_err(|e| anyhow!(e))
            .context("unable to parse response")
            .map_err(WorkOSError::Unknown)
    }

    pub async fn get_groups(
        &self,
        group_ids: Vec<String>,
    ) -> Result<HashMap<String, Group>, WorkOSError> {
        stream::iter(group_ids)
            .then(|id| async move {
                let group = self.get_group(&id).await.context("unable to get group")?;
                Ok::<Group, WorkOSError>(group)
            })
            .try_fold(HashMap::new(), |mut groups, group| async move {
                groups.insert(group.id.clone(), group);
                Ok(groups)
            })
            .await
    }

    pub async fn get_organization(
        &self,
        organization_id: &str,
    ) -> Result<Organization, WorkOSError> {
        let mut res = surf::get(format!(
            "{}/organizations/{}",
            &self.base_url, organization_id
        ))
        .header("Authorization", format!("Bearer {}", &self.api_key))
        .await
        .map_err(|e| anyhow!(e))
        .context("unable to perform request")
        .map_err(WorkOSError::Unknown)?;

        match res.status() {
            StatusCode::Ok => Ok(()),
            StatusCode::NotFound => Err(WorkOSError::NotFound(organization_id.to_string())),
            _ => Err(WorkOSError::Unknown(anyhow!(format!(
                "received unexpected status '{}'",
                res.status()
            )))),
        }?;

        res.body_json()
            .await
            .map_err(|e| anyhow!(e))
            .context("unable to parse response")
            .map_err(WorkOSError::Unknown)
    }
}
